#$Id$#

from books.model.Transaction import Transaction
from books.model.TransactionList import TransactionList
from books.model.PageContext import PageContext
from books.model.Instrumentation import Instrumentation
from books.model.SearchCriteria import SearchCriteria

class BankTransactionsParser:
    """This class is used to parse the json response for Bank transactions."""
   
    def get_list(self, resp):
        """This method parses the given response and returns bank transaction 
            list.

        Args: 
            resp(dict): Response containing json object for Bank transaction 
                list.

        Returns:
            instance: Bank transaction list object.

        """
        bank_transaction_list = TransactionList()
        for value in resp['banktransactions']:
            bank_transaction = Transaction()
            bank_transaction.set_transaction_id(value['transaction_id'])
            bank_transaction.set_date(value['date'])
            bank_transaction.set_amount(value['amount']) 
            bank_transaction.set_transaction_type(value['transaction_type'])
            bank_transaction.set_status(value['status'])
            bank_transaction.set_source(value['source'])
            bank_transaction.set_account_id(value['account_id'])
            bank_transaction.set_customer_id(value['customer_id'])
            bank_transaction.set_payee(value['payee'])
            bank_transaction.set_currency_id(value['currency_id'])
            bank_transaction.set_currency_code(value['currency_code'])
            bank_transaction.set_debit_or_credit(value['debit_or_credit'])
            bank_transaction.set_offset_account_name(value['offset_account_name'])
            bank_transaction.set_reference_number(value['reference_number'])
            bank_transaction.set_imported_transaction_id(value[\
            'imported_transaction_id'])
            bank_transaction_list.set_transactions(bank_transaction)
        page_context_obj = PageContext()
        page_context = resp['page_context']
        page_context_obj.set_page(page_context['page'])
        page_context_obj.set_per_page(page_context['per_page'])
        page_context_obj.set_has_more_page(page_context['has_more_page'])
        page_context_obj.set_report_name(page_context['report_name'])
        page_context_obj.set_applied_filter(page_context['applied_filter'])
        page_context_obj.set_sort_column(page_context['sort_column'])
        page_context_obj.set_sort_order(page_context['sort_order'])
        bank_transaction_list.set_page_context(page_context_obj)
        return bank_transaction_list

    def get_transaction(self, resp):
        """This method parses the given response and returns Bank Transaction 
            object.
      
        Args: 
            resp(dict): Response containing json object for Bank transaction.

        Returns:
            instance: Bank transaction object.

        """
        transactions = Transaction()
        bank_transaction = resp['banktransaction']
        transactions.set_transaction_id(bank_transaction['transaction_id'])
        transactions.set_from_account_id(bank_transaction['from_account_id'])
        transactions.set_from_account_name(bank_transaction[\
        'from_account_name'])
        transactions.set_to_account_id(bank_transaction['to_account_id'])
        transactions.set_to_account_name(bank_transaction['to_account_name'])
        transactions.set_transaction_type(bank_transaction['transaction_type'])
        transactions.set_currency_id(bank_transaction['transaction_type'])
        transactions.set_currency_code(bank_transaction['currency_code'])
        transactions.set_amount(bank_transaction['amount'])
        transactions.set_exchange_rate(bank_transaction['exchange_rate'])
        transactions.set_date(bank_transaction['date'])
        transactions.set_reference_number(bank_transaction['reference_number'])
        transactions.set_description(bank_transaction['description'])
        transactions.set_imported_transaction(bank_transaction[\
        'imported_transactions'])
        return transactions

    def get_message(self, resp):
        """This method parses the given response and returns message.

        Args:
            resp(dict): Response containing json object for message.

        Returns:
            str: Success message('The transaction hasbeen deleted.').

        """
        return resp['message']

    def get_matching_transaction(self, resp): 
        """This method parses the given response and returns matching 
            transactions list.
  
        Args: 
            resp(dict): Response containing json object for matching 
                transactions.

        Returns:
            instance: Transaction list object.

        """
        transaction_list = TransactionList()
        for value in resp['matching_transactions']:
            transaction = Transaction()
            transaction.set_transaction_id(value['transaction_id'])
            transaction.set_date(value['date'])
            transaction.set_date_formatted(value['date_formatted'] if 'date_formatted' in value else None)
            transaction.set_transaction_type(value['transaction_type'])
            transaction.set_transaction_type_formatted(value[\
            'transaction_type_formatted'] if 'transaction_type_formatted' in value else None)
            transaction.set_reference_number(value['reference_number'])
            transaction.set_amount(value['amount'])
            transaction.set_amount_formatted(value['amount_formatted'] if 'amount_formatted' in value else None)
            transaction.set_debit_or_credit(value['debit_or_credit'])
            transaction.is_best_match = value['is_best_match'] if 'is_best_match' in value else None
            transaction.offset_account_name = value['offset_account_name'] if 'offset_account_name' in value else None
            transaction_list.set_transactions(transaction)
        page_context_obj = PageContext()
        page_context = resp['page_context']
        page_context_obj.set_page(page_context['page'])
        page_context_obj.set_per_page(page_context['per_page'])
        page_context_obj.set_has_more_page(page_context['has_more_page'])
        page_context_obj.set_sort_column(page_context['sort_column'])
        page_context_obj.set_sort_order(page_context['sort_order'])
        for value in page_context['search_criteria']:
            criteria = SearchCriteria()
            criteria.set_column_name(value['column_name'])
            if value['comparator'] != 'range':
                criteria.set_search_text(value['search_text'])
            else:
                criteria.set_search_text('{} -- {}'.format(value['search_text_0'], value['search_text_0']))
            criteria.set_comparator(value['comparator'])
            page_context_obj.set_search_criteria(criteria)
        transaction_list.set_page_context(page_context_obj)
        '''instrumentation_obj = Instrumentation()
        instrumentation = resp['instrumentation']
        instrumentation_obj.set_query_execution_time(instrumentation[\
        'query_execution_time'])
        instrumentation_obj.set_request_handling_time(instrumentation[\
        'request_handling_time'])
        instrumentation_obj.set_response_write_time(instrumentation[\
        'response_write_time'])
        instrumentation_obj.set_page_context_write_time(instrumentation[\
        'page_context_write_time'])
        transaction_list.set_instrumentation(instrumentation_obj)'''
        return transaction_list

    def get_associated_transaction(self, resp):
        """This method parses the given response and returns associated 
            transactions list.

        Args:
            resp(dict): Response containing json object for associated 
                transactions.

        Returns:
            instance: Transaction object.

        """
        transaction = resp['transaction']
        transaction_obj = Transaction()
        transaction_obj.set_imported_transaction_id(transaction[\
        'imported_transaction_id'])
        transaction_obj.set_date(transaction['date'])
        transaction_obj.set_amount(transaction['amount'])
        transaction_obj.set_payee(transaction['payee'])
        transaction_obj.set_reference_number(transaction['reference_number'])
        transaction_obj.set_description(transaction['description'])
        transaction_obj.set_status(transaction['status'])
        transaction_obj.set_status_formatted(transaction['status_formatted'])
        for value in transaction['associated_transactions']:
            transaction = Transaction() 
            transaction.set_transaction_id(value['transaction_id'])
            transaction.set_date(value['date'])
            transaction.set_debit_or_credit(value['debit_or_credit'])
            transaction.set_transaction_type(value['transaction_type'])
            transaction.set_amount(value['amount'])
            transaction.set_customer_id(value['customer_id'])
            transaction.set_customer_name(value['customer_name'])
            transaction_obj.set_associated_transaction(transaction)
        page_context_obj = PageContext()
        page_context = resp['page_context']
        page_context_obj.set_page(page_context['page'])
        page_context_obj.set_per_page(page_context['per_page'])
        page_context_obj.set_has_more_page(page_context['has_more_page'])
        '''instrumentation_obj = Instrumentation()
        instrumentation = resp['instrumentation']
        instrumentation_obj.set_query_execution_time(instrumentation[\
        'query_execution_time'])
        instrumentation_obj.set_request_handling_time(instrumentation[\
        'request_handling_time'])
        instrumentation_obj.set_response_write_time(instrumentation[\
        'response_write_time'])
        instrumentation_obj.set_page_context_write_time(instrumentation[\
        'page_context_write_time'])
        transaction_obj.set_instrumentation(instrumentation_obj)'''
        return transaction_obj

    
            





#$Id$

class NotesAndTerms:
    """This class is used to create object for notes and terms."""
    def __init__(self):
        """Initialize parameters for notes and terms."""
        self.notes = ''
        self.terms = ''
 
    def set_notes(self, notes):
        """Set notes.

        Args:
            notes(str): Notes.

        """
        self.notes = notes

    def get_notes(self):
        """Get notes.

        Returns:
            str: Notes.

        """
        return self.notes

    def set_terms(self, terms):
        """Set terms.

        Args:
            terms(str): Terms.

        """
        self.terms = terms

    def get_terms(self):
        """Get terms.

        Returns:
            str: Terms.

        """
        return self.terms

    def to_json(self):
        """This method is used to convert notes and terms object to json form.
 
        Returns:
            dict: Dictionary containing json object for notes and terms.

        """
        data = {} 
        if self.notes != '':
            data['notes'] = self.notes
        if self.terms != '':
            data['terms'] = self.terms
        return data



#$Id$

class InvoicePayment:
    """This class is used to create object for Invoice Payments."""
    def __init__(self):
        """Initialize parameters for Invoice payments."""
        self.invoice_payment_id = ''  
        self.payment_id = ''
        self.invoice_id = ''
        self.amount_used = 0.0
        self.amount_applied = 0.0 
        self.payment_number = ''
        self.payment_mode = ''
        self.description = ''
        self.date = ''
        self.reference_number = ''
        self.exchange_rate = 0.00
        self.amount = 0.00
        self.tax_amount_withheld = 0.0
        self.is_single_invoice_payment = None
    
    def set_invoice_payment_id(self, invoice_payment_id):
        """Set invoice payment id.

        Args:
            invoice_payment_id(str): Invoice payment id.

        """
        self.invoice_payment_id = invoice_payment_id

    def get_invoice_payment_id(self):
        """Get invoice payment id.

        Returns:
            str: Invoice payment id.

        """
        return self.invoice_payment_id
   
    def set_invoice_id(self, invoice_id):
        """Set invoice id.

        Args: 
            invoice_id(str): Invoice id.

        """
        self.invoice_id = invoice_id

    def get_invoice_id(self):
        """Get invoice id.

        Returns:
            str: Invoice id.

        """
        return self.invoice_id     

    def set_payment_id(self, payment_id):
        """Set payment id.

        Args:
            payment_id(str): Payment id.

        """
        self.payment_id = payment_id

    def get_payment_id(self):
        """Get payment id.

        Returns:
            str: Payment id.

        """
        return self.payment_id
    
    def set_amount_applied(self, amount_applied):
        """Set amount applied.
 
        Args:
            amount_applied(float): Amount applied.

        """
        self.amount_applied = amount_applied

    def get_amount_applied(self):
        """Get amount applied.

        Returns:
            float: Amount applied.

        """
        return self.amount_applied

    def set_amount_used(self, amount_used):
        """Set amount used.

        Args:
            amount_used(float): Amount used.

        """
        self.amount_used = amount_used
 
    def get_amount_used(self): 
        """Get amount used.

        Returns:
            float: Amount used.

        """
        return self.amount_used

    def set_payment_number(self, payment_number):
        """Set payment number.

        Args:
            payment_number(str): Payment number.

        """
        self.payment_number = payment_number

    def get_payment_number(self): 
        """Get payment number.

        Returns:
            str: Payment number.

        """
        return self.payment_number
   
    def set_payment_mode(self, payment_mode):
        """Set payment mode.

        Args: 
            payment_mode(str): Payment mode.

        """
        self.payment_mode = payment_mode

    def get_payment_mode(self):
        """Get payment mode.

        Returns:
            str: Payment mode.

        """
        return self.payment_mode

    def set_description(self, description):
        """Set description.

        Args:
            description(str): Description.

        """
        self.description = description

    def get_description(self):
        """Get description.

        Returns:
            str: Description.

        """
        return self.description

    def set_date(self, date):
        """Set date.

        Args:
            date(str): Date.

        """
        self.date = date

    def get_date(self):
        """Get date.

        Returns:
            str: Date.
 
        """
        return self.date

    def set_reference_number(self, reference_number):
        """Set reference number.

        Args:
            reference_number(str): Reference number.

        """
        self.reference_number = reference_number

    def get_reference_number(self):
        """Get reference number.

        Returns: 
            str: Reference number.

        """
        return self.reference_number
  
    def set_exchange_rate(self, exchange_rate):
        """Set exchange rate.

        Args: 
            exchange_rate(float): Exchange rate.

        """
        self.exchange_rate = exchange_rate

    def get_exchange_rate(self):
        """Get exchange rate.

        Returns:
            float: Exchange rate.

        """
        return self.exchange_rate
    
    def set_amount(self, amount):
        """Set amount.

        Args:
            amount(float): Amount.

        """
        self.amount = amount

    def get_amount(self):
        """Get amount.
    
        Returns:
            float: Amount.
 
        """ 
        return self.amount

    def set_tax_amount_withheld(self, tax_amount_withheld):
        """Set tax amount withheld.

        Args:
            tax_amount_withheld(float): Tax amount withheld.

        """
        self.tax_amount_withheld = tax_amount_withheld

    def get_tax_amount_withheld(self):
        """Get tax amount withheld.

        Returns:
            float: Tax amount withheld.

        """
        return self.tax_amount_withheld
 
    def set_is_single_invoice_payment(self, is_single_invoice_payment):
        """Set whether it is single invoice payment.

        Args:
            is_single_invoice_payment(bool): True if it is single invoice 
                 payment else False.

        """
        self.is_single_invoice_payment = is_single_invoice_payment

    def get_is_single_invoice_payment(self):
        """Get whether it is single invoice payment.

        Returns:
            bool: True if it is single invoice payment else False.

        """
        return self.is_single_invoice_payment
  



#$Id$

from books.model.PageContext import PageContext

class TimeEntryList:
    """This class is used to create object for Time entrie list."""
    def __init__(self):
        """Initialize parameters for time entries list."""
        self.time_entries = []
        self.page_context = PageContext()

    def set_time_entries(self, time_entry):
        """Set time entries.

        Args:
            time_entry(instance): Time entry object.

        """
        self.time_entries.append(time_entry)
  
    def get_time_entries(self):
        """Get time entries.

        Returns:
            instance: Time entry object.

        """
        return self.time_entries

    def set_page_context(self, page_context):
        """Set page context.

        Args:
            page_context(instance): Page context object.

        """
        self.page_context = page_context

    def get_page_context(self):
        """Get page context.

        Returns:
            instance: Page context object.

        """
        return self.page_context

   



#$Id$

class Journal:   
    """This class is used to create object for Journals."""
    def __init__(self): 
        """Initilaize parameters for journals object."""
        self.journal_date = ''
        self.reference_number = ''
        self.notes = ''
        self.line_items = []
        self.journal_id = ''
        self.entry_number = ''
        self.currency_id = ''
        self.currency_symbol = ''
        self.journal_date = ''
        self.line_item_total = 0.0
        self.total = 0.0
        self.price_precision = 0
        self.taxes = []
        self.created_time = ''
        self.last_modified_time = ''

    def set_journal_date(self, journal_date):
        """Set journal date.
 
        Args:
            journal_date(str): Journal date.
 
        """
        self.journal_date = journal_date

    def get_journal_date(self):
        """Get journal date.

        Returns:
            str: Journal date.

        """
        return self.journal_date

    def set_reference_number(self, reference_number):
        """Set reference date.

        Args:
            reference_number(str): Reference number.

        """
        self.reference_number = reference_number

    def get_reference_number(self):
        """Get reference number.

        Returns:
            str: Reference number.

        """
        return self.reference_number

    def set_notes(self, notes):
        """Set notes.

        Args:
            notes(str): Notes.

        """
        self.note = notes

    def get_notes(self):
        """Get notes.

        Returns:
            str: Notes.

        """
        return self.notes

    def set_line_items(self, line_item):
        """Set line items.

        Args:
            line_items(instance): Line items.

        """
        self.line_items.append(line_item)

    def get_line_items(self):
        """Get line items.

        Returns:
            list of instance: List of instance object.

        """
        return self.line_items

    def set_journal_id(self, journal_id): 
        """Set journal id.

        Args:
            journal_id(str): Journal id.

        """
        self.journal_id = journal_id

    def get_journal_id(self):
        """Get journal id.
 
        Returns:
            str: Journal id.

        """
        return self.journal_id

    def set_entry_number(self, entry_number): 
        """Set entry number.

        Args:
            entry_number(str): Entry number.

        """
        self.entry_number = entry_number

    def get_entry_number(self):
        """Get entry number.

        Returns:
            str: Entry number.

        """
        return self.entry_number

    def set_currency_id(self, currency_id): 
        """Set currecny id.

        Args:
            currency_id(str): Currency id.

        """
        self.currency_id = currency_id

    def get_currency_id(self):
        """Get currency id.

        Returns: 
            str: Currency id.

        """
        return self.currency_id

    def set_currency_symbol(self, currency_symbol): 
        """Set currency symbol.

        Args:
            currency_symbol(str): Currency symbol.

        """
        self.currency_symbol = currency_symbol

    def get_currency_symbol(self):
        """Get currency symbol.

        Returns:
            str: Currency symbol.

        """
        return self.currency_symbol

    def set_journal_date(self, journal_date):
        """Set journal date.

        Args: 
            journal_date(str): Journal date.

        """ 
        self.journal_date = journal_date

    def get_journal_date(self):
        """Get journal date.

        Returns:
            str: Journal date.

        """
        return self.journal_date

    def set_line_item_total(self, line_item_total):
        """Set line item total.

        Args:
            lin_item_total(float): Line item total.

        """
        self.line_item_total = line_item_total

    def get_line_item_total(self):
        """Get line item total.

        Returns:
            float: Line item total.

        """
        return self.line_item_total

    def set_total(self, total): 
        """Set total.

        Args:
            total(float): Total.

        """
        self.total = total

    def get_total(self):
        """Get total.

        Returns: 
            float: Total.

        """
        return self.total

    def set_price_precision(self, price_precision):
        """Set price precision.

        Args:
            price_precision(int): Price precision.

        """
        self.price_precision = price_precision

    def get_price_precision(self):
        """Get price precision.

        Returns:
            int: Price precision.

        """
        return self.price_precision

    def set_taxes(self, tax):
        """Set taxes.

        Args:
            tax(instance): Tax.

        """
        self.taxes.append(tax)
  
    def get_taxes(self):
        """Get taxes.

        Returns:
            list of instance: List of tax object.

        """
        return self.taxes

    def set_created_time(self, created_time):
        """Set created time.

        Args:
            created_time(str): Created time.

        """
        self.created_time = created_time

    def get_created_time(self):
        """Get created time.

        Returns: 
            str: Created time.

        """
        return self.created_time

    def set_last_modified_time(self, last_modified_time):
        """Set last modified time.

        Args:
            last_modified_time(str): Last modified time.

        """
        self.last_modified_time = last_modified_time

    def get_last_modified_time(self): 
        """Get last modified time.

        Returns:
            str: Last modified time.

        """
        return self.last_modified_time

    def to_json(self): 
        """This method is used to create json object for journals.

        Returns:
            dict: Response containing json object for journals.

        """
        data = {} 
        if self.journal_date != '':       
            data['journal_date'] = self.journal_date
        if self.reference_number != '':
            data['reference_number'] = self.reference_number
        if self.notes != '':
            data['notes'] = self.notes
        if self.line_items:
            data['line_items'] = []
            for value in self.line_items:
                line_item = value.to_json()
                data['line_items'].append(line_item)
        return data 




#$Id$

class TaxGroup:
    """This class is used to create object for tax group."""
    def __init__(self):
        """Initialize parameters for tax group. """
        self.tax_group_id = ''
        self.tax_group_name = ''
        self.tax_group_percentage = 0
        self.taxes = ''

    def set_tax_group_id(self, tax_group_id):
        """Set tax group id.

        Args:
            tax_group_id(str): Tax group id.

        """
        self.tax_group_id = tax_group_id

    def get_tax_group_id(self):
        """Get tax group id.

        Returns:
            str: Tax group id.

        """
        return self.tax_group_id

    def set_tax_group_name(self, tax_group_name):
        """Set tax group name.

        Args:
            tax_group_name(str): Tax group name.

        """
        self.tax_group_name = tax_group_name

    def get_tax_group_name(self):
        """Get tax group name.

        Returns:
            str: Tax group name.

        """
        return self.tax_group_name

    def set_tax_group_percentage(self, tax_group_percentage):
        """Set tax group percentage.

        Args:
            tax_group_percentage(int): Tax group percentage.

        """ 
        self.tax_group_percentage = tax_group_percentage

    def get_tax_group_percentage(self):
        """Get tax group percentage.

        Returns:
            int: Tax group percentage.

        """
        return self.tax_group_percentage

    def set_taxes(self, taxes):
        """Set taxes.

        Args:
            taxes(str): Taxes.

        """
        self.taxes = taxes

    def get_taxes(self):
        """Get taxes.

        Returns:
            str: Taxes.

        """ 
        return self.taxes
 
    def to_json(self):
        """This method is used to create json object for tax group.

        Returns:
            dict: Dictionary containing json object for tax group.

        """
        data = {}
        if self.tax_group_name != '':
            data['tax_group_name'] = self.tax_group_name
        if self.taxes != '':
            data['taxes'] = self.taxes
        return data



#$Id$

from books.model.PageContext import PageContext

class JournalList:
    """This class is used to create object for journals list."""
    def __init__(self):
        """Initialize parameters for Journals list object."""
        self.journals = []
        self.page_context = PageContext()

    def set_journals(self, journal):
        """Set journals.

        Args:
            journal(instance): Journal object.

        """
        self.journals.append(journal)

    def get_journals(self):
        """Get journals.

        Returns:
            list of instance: List of journals object.

        """
        return self.journals

    def set_page_context(self, page_context):
        """Set page context.

        Args:
            page_context(instance): Page context object.

        """
        self.page_context = page_context

    def get_page_context(self): 
        """Get page context.

        Returns:
            instance: Page context object.

        """
        return self.page_context

 




#$Id$#

from os.path import basename
from json import dumps
from books.util.ZohoHttpClient import ZohoHttpClient
from books.parser.ContactParser import ContactParser
from .Api import Api

base_url = Api().base_url + 'contacts/'
parser = ContactParser()
zoho_http_client = ZohoHttpClient()

class ContactsApi:
    """ContactsApi class is used:

    1.To get the list of contacts for a particular organization.
    2.To get details of particular contact.
    3.To create a new contact for an organization.
    4.To update a contact.
    5.To delete a contact.
    6.To mark a contact as active.
    7.To mark a contact as inactive.
    8.To enable payment reminders for a contact
    9.To disable payment reminders for a contact.
    10.To send email statement to a contact.
    11.To get the statement mail content
    12.To send email to a contact.
    13.To list recent activities of a contact.
    14.To list the refund history of a contact.
    15.To track a contact for 1099 reporting.
    16.To untrack a contact for 1099 reporting.

    """
    def __init__(self, authtoken, organization_id):
        """Initialize Contacts Api using user's authtoken and organization id.

        Args:
            authtoken(str): User's authtoken.
            organization_id(str): User's organization id.

        """
        self.details={
            'authtoken': authtoken,
            'organization_id': organization_id
            }
  
    def get_contacts(self, parameter=None):   
        """List of contacts with pagination for a particular organization.

        Args:
            parameter(dict, optional): Filter with which the list has to be 
                displayed. Defaults to None.

        Returns:
            instance: List of contact objects with pagination.

        """
        response = zoho_http_client.get(base_url, self.details, parameter)
        return parser.get_contacts(response) 

    def get(self, contact_id):
        """Get details of a contact.

        Args:
            contact_id(str): Contact_id of a particular contact.

        Returns:
            instance: Contact object.

        """
        url = base_url + contact_id
        response = zoho_http_client.get(url, self.details, None)
        return parser.get_contact(response) 

    def create(self, contact):
        """Create a contact.

        Args:
            contact(instance): Contact object.

        Returns:
            instance: Contact object.

        """
        data = contact.to_json()
        field = {
                'JSONString':dumps(data)
                } 
        response = zoho_http_client.post(base_url, self.details, field, None)
        return parser.get_contact(response) 
   
    def update(self, contact_id, contact):
        """Update a contact with the given information.

        Args:
            contact_id(str): Contact_id of the contact that has to be updated.
            contact(instance): Contact object which has the information that 
                has to be updated.

        Returns:
            instance: Contact object.

        Raises:
            Books exception: If status is not '200' or '201'.

        """
        url = base_url + contact_id
        data = contact.to_json()
        field = {
                'JSONString':dumps(data)
                } 
        response = zoho_http_client.put(url, self.details, field, None)
        return parser.get_contact(response) 
 
    def delete(self, contact_id):
        """Delete a particular contact.

        Args:
            contact_id(str): Contact id of the contact to be deleted.

        Returns:
            str: Success message('The contact has been deleted').

        Raises: 
            Books exception: If status is not '200' or '201'.

        """ 
        url = base_url +  contact_id
        response = zoho_http_client.delete(url, self.details)
        return parser.get_message(response) 

    def mark_active(self, contact_id):
        """Mark a contact as active.

        Args:
            contact_id(str): Contact id of the contact.

        Returns:
            str: Success message('The contact has been marked as active').

        Raises:
            Books exception: If status is not '200' or '201'.

        """
        url= base_url + contact_id + '/active'
        response = zoho_http_client.post(url, self.details, '')
        return parser.get_message(response) 

    def mark_inactive(self, contact_id):
        """Mark a contact as inactive.

        Args:
            contact_id(str): Contact id of the contact.

        Returns:
            str: Success message('The contact has been marked as inactive').

        Raises:
            Books exception: If status is not '200' or '201'.

        """         
        url = base_url + contact_id + '/inactive'
        response = zoho_http_client.post(url, self.details, '')
        return parser.get_message(response) 

    def enable_payment_reminder(self, contact_id):
        """Enable automated payment reminders for a contact.

        Args:
            contact_id(str): Contact id of the contact.

        Returns:
            str: Success message('All reminders associated with this contact 
                have been enabled').

        Raises:
            Books exception: If status is not '200' or '201'.

        """
        url = base_url + contact_id + '/paymentreminder/enable'
        response = zoho_http_client.post(url, self.details, '')
        return parser.get_message(response)
 
    def disable_payment_reminder(self, contact_id):
        """Disable automated payment reminders for a contact.

        Args:
            contact_id(str): Contact id of the contact.

        Returns:
            str: Success message('All reminders associated with this contact 
                have been disabled').

        Raises:
            Books exception: If status is not '200' or '201'.

        """
        url = base_url + contact_id + '/paymentreminder/disable'
        response = zoho_http_client.post(url, self.details, '')
        return parser.get_message(response) 

    def email_statement(self, contact_id, email,start_date=None, end_date=None, 
                        attachments=None):
        """Email statement to the contact. If JSONString is not inputted, mail 
            will be sent with the default mail content.

        Args:
            contact_id(str): Contact id of the contact.
            email(instance): Email.
            start_date(str, optional): Starting date of the statement. 
                Default to None.
            end_date(str, optional): Ending date of the statement. 
                Default to None.
                If starting date and ending date is not given current month's 
                    statement will be sent to the contact.
            attachments(list): List of files to be attached.

        Returns:
            str: Success message('Statement has to been sent to the customer').

        Raises:
            Books exception: If status is not '200' or '201'.

        """
        url = base_url + contact_id + '/statements/email'
        data = {}
        data = email.to_json()
        if start_date is not None and end_date is not None:
            data['start_date'] = start_date
            data['end_date'] = end_date
        fields = {
            'JSONString': dumps(data)
                }
        if attachments is None:
            response = zoho_http_client.post(url, self.details, fields)
        else:
            file_list = []

            for value in attachments:
                attachment = {
                    'attachments': {
                        'filename': basename(value), 
                        'content': open(value).read()
                        }
                    }
                file_list.append(attachment)
            
            response = zoho_http_client.post(url, self.details, fields, 
                                                     None, file_list)
        return parser.get_message(response) 

    def get_statement_mail_content(self, contact_id, start_date, end_date):
        """Get the statement mail content.

        Args:
            start_date(str): Start date for statement.
            end_date(str): End date for statement.

        Returns:
            instance: Email object.

        Raises: 
            Books exception:if status is not '200' or '201'.

        """ 
        url = base_url + contact_id + '/statements/email'
        query_string = {
            'start_date': start_date,
            'end_date': end_date
            }
        response = zoho_http_client.get(url, self.details, query_string)
        return parser.get_mail_content(response) 
 
    def email_contact(self, contact_id, email, attachments=None, 
                      send_customer_statement=None):
        """Send email to contact.

        Args:
            contact_id(str): Contact id of the contact.
            email(instance): Email object.
            attachments(list, optional): List of files to be attached. 
                Default to None.
            send_customer_statement(bool, optional): Send customer statement 
                pdf with email. Default to None.

        Returns:
            str: Success message('Email has been sent').

        Raises:
            Books exception: If status is not '200' or '201'.

        """
        url = base_url + contact_id + '/email'

        json_object = dumps(email.to_json())
        data = {
            'JSONString': json_object
            }
        if attachments is not None and send_customer_statement is not None:
            file_list = []

            for value in attachments:
                attachment = {
                    'attachments': {
                        'filename': basename(value), 
                        'content': open(value).read()
                        }
                    }
                file_list.append(attachment)

            parameter = {
                'send_customer_statement': send_customer_statement
                }
            response = zoho_http_client.post(url, self.details, data, 
                                                     parameter, file_list)

        elif attachments is not None:
            file_list = []

            for value in attachments:
                attachment = {
                    'attachments': {
                        'filename': basename(value), 
                        'content': open(value).read()
                        }
                    }
                file_list.append(attachment)

            response = zoho_http_client.post(url, self.details, 
                                                     data, None, file_list)

        elif send_customer_statement is not None:
            parameter = { 
                'send_customer_statement': send_customer_statement
                }
            response = zoho_http_client.post(url, self.details, data, 
                                                     parameter)

        else:
            response = zoho_http_client.post(url, self.details, data)

        return parser.get_message(response)

    def list_comments(self, contact_id):
        """List recent activities of a contact with pagination.

        Args:
            contact_id(str): Contact id of the contact.

        Returns:
            instance: Comments list object.

        Raises:
            Books exception: If status is not '200' or '201'.

        """
        url = base_url + contact_id + '/comments'
        response = zoho_http_client.get(url, self.details)
        return parser.get_comment_list(response) 

    def get_comments(self, contact_id):
        """List recent activities of a contact.

        Args:
            contact_id(str): Contact id of the contact.

        Returns:
            list: List of comments object.

        Raises:
            Books exception: If status is not '200' or '201'.

        """
        url = base_url + contact_id + '/comments'
        response = zoho_http_client.get(url, self.details)
        return parser.get_comment_list(response).get_comments()
     
    def list_refunds(self, contact_id):
        """List the refund history of a contact with pagination.

        Args: 
            contact_id(str): Contact id of the contact.

        Returns:
            instance: Refunds list object.

        Raises:
            Books exception: If status is not '200' or '201'.

        """
        url = base_url + contact_id + '/refunds'
        response = zoho_http_client.get(url, self.details)
        return parser.get_refund_list(response) 
 
    def get_refunds(self, contact_id): 
        """List the refund history of a contact.

        Args: 
            contact_id(str): Contact id of the contact.

        Returns:
            list: List of refunds object.

        Raises:
            Books exception: If status is not '200' or '201'.

        """
        url = base_url + contact_id + '/refunds'
        response = zoho_http_client.get(url, self.details)
        return parser.get_refund_list(response).get_creditnote_refunds() 
    
    def track_1099(self, contact_id):
        """Track a contact for 1099 reporting.

        Args:
            contact_id(str): Contact id of the contact.

        Returns:
            str: Success message('1099 tracking is enabled').

        Raises: 
            Books exception: If status is not '200' or '201'.

        """
        url = base_url + contact_id + '/track1099'
        response = zoho_http_client.post(url, self.details, '')
        return parser.get_message(response)   

    def untrack_1099(self, contact_id):
        """Track a contact for 1099 reporting.

        Args:
            contact_id(str): Contact id of the contact.

        Returns:
            str: Success message('1099 tracking is enabled').

        Raises: 
            Books exception: If status is not '200' or '201'.

        """
        url = base_url + contact_id + '/untrack1099'
        response = zoho_http_client.post(url, self.details, '')
        return parser.get_message(response) 



#$Id$

from books.model.PageContext import PageContext

class ChartOfAccountList:
    """This class is used to create object for Chart of accounts list."""
    def __init__(self):
        """Initialize parameters for chart of accounts list object."""
        self.chartofaccounts = []
        self.page_context = PageContext()

    def set_chartofaccounts(self, chart_of_accounts):
        """Set chart of accounts.

        Args:
            chart_of_accounts(instance): Chart of accounts object.

        """
        self.chartofaccounts.append(chart_of_accounts)

    def get_chartofaccounts(self):
        """Get chart of accounts.

        Returns:
            list of instance: List of chart of accounts object.

        """
        return self.chartofaccounts

    def set_page_context(self, page_context):
        """Set page context.

        Args:
            page_context(instance): Page context object.

        """
        self.page_context = page_context

    def get_page_context(self):
        """Get page context.

        Returns:
            instance: Page context object.

        """
        return self.page_context




#$Id$

class ContactPerson:
    """This class is used to create model for contact person."""
    def __init__(self):
        """Initialize the parameters for contact person."""
        self.contact_person_id = ''
        self.salutation = ''
        self.first_name = ''
        self.last_name = ''
        self.email = ''
        self.phone = ''
        self.mobile = ''
        self.is_primary_contact = None
        self.contact_id = ''

    def set_contact_person_id(self, contact_person_id):
        """Set contact person id.
 
        Args: 
            contact_person_id(str): Contact person id.

        """
        self.contact_person_id = contact_person_id

    def get_contact_person_id(self):
        """Get contact person id.
 
        Returns:
            str: Contact person id.

        """
        return self.contact_person_id
  
    def set_salutation(self, salutation):
        """Set salutation.
   
        Args:
            salutation(str): Salutation.

        """
        self.salutation = salutation

    def get_salutation(self):
        """Get salutation.
        
        Returns:
            str: Salutation.
 
        """
        return self.salutation
  
    def set_first_name(self, first_name):
        """Set first name.
 
        Args: 
            first_name(str): First name.

        """
        self.first_name = first_name

    def get_first_name(self):
        """Get first name.

        Returns: 
            str: First name.

        """
        return self.first_name
 
    def set_last_name(self, last_name):
        """Set last name.
        
        Args:    
            last_name(str): Last name.

        """  
        self.last_name = last_name

    def get_last_name(self):
        """Get last name.
        
        Returns:
            str: Last name.

        """
        return self.last_name

    def set_email(self, email):
        """Set email.
   
        Args:
            email(str): Email.

        """
        self.email = email

    def get_email(self):
        """Get email.
 
        Returns: 
            str: Email.

        """
        return self.email
  
    def set_phone(self, phone):
        """Set phone.
   
        Args:
            phone(str): Phone.

        """
        self.phone = phone

    def get_phone(self):
        """Get phone.

        Returns:
            str: Phone.
       
        """
        return self.phone
 
    def set_mobile(self, mobile):
        """Set mobile.

        Args:
            mobile(str): Mobile.

        """
        self.mobile = mobile

    def get_mobile(self):
        """Get mobile.

        Args: 
            mobile(str): Mobile.

        """
        return self.mobile
  
    def set_is_primary_contact(self, is_primary_contact):
        """Set whether it is primary contact or not.

        Args:
            is_primary_contact(bool): True if it is primary contact. 
                Allowed value is true only.

        """
        self.is_primary_contact = is_primary_contact

    def get_is_primary_contact(self):
        """Get whether it is primary contact or not.

        Returns:
            bool: True if it os primary contact else False.

        """
        return self.is_primary_contact

    def set_contact_id(self, contact_id):
        """Set contact id.
 
        Args:
            contact_id(str): Contact id.

        """
        self.contact_id = contact_id

    def get_contact_id(self):
        """Get contact id.

        Returns: 
            str: Contact id.

        """
        return self.contact_id

    def to_json(self):
        """This method is used to convert the contact person object to JSON object.

        Returns:
            dict: Dictionary containing details of contact person object.

        """
        contact_person = {}
        if self.salutation != '':
            contact_person['salutation'] = self.salutation
        if self.first_name != '':
            contact_person['first_name'] = self.first_name
        if self.last_name != '':
            contact_person['last_name'] = self.last_name
        if self.email != '':
            contact_person['email'] = self.email
        if self.phone != '':
            contact_person['phone'] = self.phone
        if self.mobile != '':
            contact_person['mobile'] = self.mobile
        if self.is_primary_contact != False:
            contact_person['is_primary_contact'] = str(\
            self.is_primary_contact).lower()
        if self.contact_id != '': 
            contact_person['contact_id'] = self.contact_id
        return contact_person



#$Id$#

from books.util.ZohoHttpClient import ZohoHttpClient
from books.parser.BaseCurrencyAdjustmentParser import BaseCurrencyAdjustmentParser
from books.api.Api import Api
from json import dumps

base_url = Api().base_url + 'basecurrencyadjustment/'
zoho_http_client = ZohoHttpClient()
parser = BaseCurrencyAdjustmentParser()

class BaseCurrencyAdjustmentApi:
    """Base Currency Adjsutment Api is used to:
    
    1.List base currency adjustment.
    2.Get base currency adjustment details.
    3.List account details for base currency adjustment.
    4.Creates a base currency adjustment.
    5.Deletes a base currency adjustment.

    """
 
    def __init__(self, authtoken, organization_id):
        """Initialize parameters for Base currency adjustment api.

        Args: 
            authotoken(str): User's Authtoken.
            organization id(str): User's Organization id.

        """
        self.details = {
            'authtoken': authtoken, 
            'organization_id': organization_id
            }

    def get_base_currency_adjustments(self, parameters=None):
        """List base currency adjustment.

        Args:
            parameters(dict, optional): Filter with whichj the list has to 
                be displayed. Defaults to None.

        Returns:
            instance: Base currency adjustment list object.

        """
        resp = zoho_http_client.get(base_url, self.details, parameters) 
        return parser.get_list(resp) 

    def get(self, base_currency_adjustment_id):
        """Get base currency adjustment details.

        Args:
            base_currency_adjustment_id(str): Base currency adjustment id.

        Returns: 
            instance: Base currency adjustment object.

        """
        url = base_url + base_currency_adjustment_id
        resp = zoho_http_client.get(url, self.details)
        return parser.get_base_currency_adjustment(resp) 

    def list_account_details(self, parameters):
        """List account details for base currency adjustments.

        Args:
            parameters(dict): Parameters with which the list has to be 
                displayed.

        Returns:
            instance: Base currency adjustment object.

        """
        url = base_url + 'accounts'
        resp = zoho_http_client.get(url, self.details, parameters)
        return parser.list_account_details(resp)

    def create(self, base_currency_adjustment, account_id): 
        """Create base currency adjustment.

        Args: 
            base_currency_adjustment(instance): Base currency adjustment 
                object.
            account_ids(str): Account ids.

        Returns:
            instance: Base currency adjustment object.

        """
        json_object = dumps(base_currency_adjustment.to_json())
        data = {
            'JSONString': json_object
            }
        account_ids = {
            'account_ids': account_id
            }
        resp = zoho_http_client.post(base_url, self.details, data, account_ids)
        return parser.get_base_currency_adjustment(resp) 

    def delete(self, base_currency_adjustment_id):
        """Delete an existing base currency adjustment.

        Args:
            base_currency_adjustment_id(str): Base currency adjustment id.

        Returns:
            str: Success message('The selected base currency adjustment has 
                been deleted.').
 
        """
        url = base_url + base_currency_adjustment_id
        resp = zoho_http_client.delete(url, self.details)
        return parser.get_message(resp)

